@@ -40,6 +40,7 @@ use strict;
 use warnings;
 
 use App::CELL qw( $CELL $log );
+use App::Dochazka::REST::Model::Employee;
 use Data::Dumper;
 use Params::Validate qw( :all );
 
@@ -55,11 +56,11 @@ App::Dochazka::REST::Dispatch::ACL - ACL module
 
 =head1 VERSION
 
-Version 0.458
+Version 0.462
 
 =cut
 
-our $VERSION = '0.458';
+our $VERSION = '0.462';
 
 
 
@@ -79,8 +80,13 @@ This module provides helper code for ACL checks.
 =cut
 
 use Exporter qw( import );
-our @EXPORT_OK = qw( check_acl check_acl_context acl_check_is_me acl_check_iid_lid );
-
+our @EXPORT_OK = qw( 
+    check_acl 
+    check_acl_context 
+    acl_check_is_me 
+    acl_check_is_my_report
+    acl_check_iid_lid
+);
 
 
 
@@ -199,27 +205,55 @@ sub acl_check_is_me {
 }
 
 
-=head2 acl_check_iid_lid
+=head2 acl_check_is_my_report
 
-When a user attempts to access an interval by IID, or a lock by LID, this
-is an additional ACL check to make sure non-admin users cannot access intervals
-and locks belonging to other users.
+Takes a property and a value. The property can be 'eid', 'nick', or 'sec_id'.
+This routine first gets the employee object corresponding to the
+eid/nick/sec_id and then checks if the current employee is that
+employee's supervisor.
 
 =cut
 
-sub acl_check_iid_lid {
-    my ( $self, $eid ) = @_;
-    my $context = $self->context;
-    my $current_eid = $context->{'current'}->{'eid'};
-    my $current_priv = $context->{'current_priv'};
-    if (   ( $current_priv eq 'passerby' ) or 
-           ( $current_priv eq 'inactive' ) or
-           ( $current_priv eq 'active' and $current_eid != $eid )
-    ) {
-        $self->mrest_declare_status( code => 403, explanation => 'DISPATCH_KEEP_TO_YOURSELF' );
+sub acl_check_is_my_report {
+    my $self = shift;
+    my %pl = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::acl_check_is_my_report with " . Dumper( \%pl ) );
+
+    my $ce = $self->context->{'current_obj'};
+    my $priv = $self->context->{'current_priv'};
+    my $emp = App::Dochazka::REST::Model::Employee->spawn;
+    my $conn = $self->context->{'dbix_conn'};
+    my $status;
+
+    return 1 if $priv eq 'admin';
+
+    if ( my $eid = $pl{'eid'} ) {
+        $log->debug( "acl_check_is_my_report: given EID $eid" );
+        $status = $emp->load_by_eid( $conn, $eid );
+    } elsif ( my $nick = $pl{'nick'} ) {
+        $log->debug( "acl_check_is_my_report: given nick $nick" );
+        $status = $emp->load_by_nick( $conn, $nick );
+    } elsif ( my $sec_id = $pl{'sec_id'} ) {
+        $log->debug( "acl_check_is_my_report: given sec_id $sec_id" );
+        $status = $emp->load_by_sec_id( $conn, $sec_id );
+    } else {
+        die "AAAGAAHHAHAHAAJJAJAJAHAHA! " . Dumper( \%pl );
+    }
+
+    if ( $status->not_ok ) {
+        $log->error( "acl_check_is_my_report: employee lookup failed (" . $status->text . ")" );
         return 0;
     }
-    return 1;
+
+    $emp = $status->payload;
+    
+    if ( $emp->supervisor eq $ce->eid ) {
+        $log->debug( "acl_check_is_my_report: I am the supervisor of ->" . $emp->nick . "<-" );
+        return 1;
+    }
+
+    return 0;
 }
 
+
 1;
