@@ -69,11 +69,11 @@ App::Dochazka::REST::Auth - HTTP request authentication and authorization
 
 =head1 VERSION
 
-Version 0.458
+Version 0.462
 
 =cut
 
-our $VERSION = '0.458';
+our $VERSION = '0.462';
 
 
 
@@ -280,7 +280,9 @@ sub _authenticate {
                     }
                     $log->notice( "Auto-created employee $nick, who was authenticated via LDAP" );
                     if ( my $priv = $site->LDAP_LDAP_AUTOCREATE_AS ) {
-                        if ( $priv =~ m/^(passerby)|(inactive)|(active)$/ ) {
+                        if ( $priv = 'passerby' ) {
+                            # do nothing
+                        } elsif ( $priv =~ m/^(inactive)|(active)$/ ) {
                             init_timepiece();
                             my $ph_obj = App::Dochazka::REST::Model::Privhistory->spawn(
                                 eid => $emp->eid,
