@@ -41,7 +41,11 @@ use warnings;
 
 use App::CELL qw( $CELL $log $core $meta $site );
 use App::Dochazka qw( $today init_timepiece );
-use App::Dochazka::REST::ACL qw( check_acl_context acl_check_is_me acl_check_iid_lid );
+use App::Dochazka::REST::ACL qw( 
+    check_acl_context 
+    acl_check_is_me 
+    acl_check_is_my_report 
+);
 use App::Dochazka::REST::ConnBank qw( conn_status );
 use App::Dochazka::REST::Model::Activity;
 use App::Dochazka::REST::Model::Employee qw( 
@@ -86,11 +90,11 @@ App::Dochazka::REST::Dispatch - Implementation of top-level resources
 
 =head1 VERSION
 
-Version 0.458
+Version 0.462
 
 =cut
 
-our $VERSION = '0.458';
+our $VERSION = '0.462';
 
 
 =head1 DESCRIPTION
@@ -1459,7 +1463,23 @@ sub handler_get_interval_eid {
     my ( $self, $pass ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::handler_get_interval_eid " ); 
 
-    return $self->_handler_get_interval( 'eid', $pass );
+    return $self->_handler_get_intlock( 'Interval', 'eid', $pass );
+}
+
+
+=head3 handler_get_lock_eid
+
+Handler for 'GET lock/eid/:eid/:tsrange'
+
+#FIXME: implement a configurable limit on the tsrange
+
+=cut
+
+sub handler_get_lock_eid {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_lock_eid " ); 
+
+    return $self->_handler_get_intlock( 'Lock', 'eid', $pass );
 }
 
 
@@ -1475,7 +1495,23 @@ sub handler_get_interval_nick {
     my ( $self, $pass ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::handler_get_interval_nick " ); 
 
-    return $self->_handler_get_interval( 'nick', $pass );
+    return $self->_handler_get_intlock( 'Interval', 'nick', $pass );
+}
+
+
+=head3 handler_get_lock_nick
+
+Handler for 'GET lock/nick/:nick/:tsrange'
+
+#FIXME: implement a configurable limit on the tsrange
+
+=cut
+
+sub handler_get_lock_nick {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_lock_nick " ); 
+
+    return $self->_handler_get_intlock( 'Lock', 'nick', $pass );
 }
 
 
@@ -1491,12 +1527,28 @@ sub handler_get_interval_self {
     my ( $self, $pass ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::handler_get_interval_self " ); 
 
-    return $self->_handler_get_interval( 'self', $pass );
+    return $self->_handler_get_intlock( 'Interval', 'self', $pass );
 }
 
 
-sub _handler_get_interval {
-    my ( $self, $key, $pass ) = @_;
+=head3 handler_get_lock_self
+
+Handler for 'GET lock/self/:tsrange'
+
+#FIXME: implement a configurable limit on the tsrange
+
+=cut
+
+sub handler_get_lock_self {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_lock_self " ); 
+
+    return $self->_handler_get_intlock( 'Lock', 'self', $pass );
+}
+
+
+sub _handler_get_intlock {
+    my ( $self, $intlock, $key, $pass ) = @_;
 
     my $context = $self->context;
 
@@ -1509,17 +1561,29 @@ sub _handler_get_interval {
         } else {
             $value = $context->{'mapping'}->{ $key };
         }
-        if ( ! acl_check_is_me( $self, $key => $value ) ) {
+        if ( 
+                ! acl_check_is_me( $self, $key => $value ) and
+                ! acl_check_is_my_report( $self, $key => $value ) 
+           ) 
+        {
             $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
             return 0;
         }
         my $emp = shared_first_pass_lookup( $self, $key, $value );
         return 0 unless $emp;
-        my $status = App::Dochazka::REST::Model::Interval::fetch_by_eid_and_tsrange(
+        my $status;
+        my @ARGS = (
             $context->{'dbix_conn'},
             $emp->eid,
             $context->{'mapping'}->{'tsrange'} 
         );
+        if ( $intlock eq 'Interval' ) {
+            $status = App::Dochazka::REST::Model::Interval::fetch_by_eid_and_tsrange( @ARGS );
+        } elsif ( $intlock eq 'Lock' ) {
+            $status = App::Dochazka::REST::Model::Lock::fetch_by_eid_and_tsrange( @ARGS );
+        } else {
+            die "AGACHCH!! Horrible, horrible: " . ( $intlock || "undef" );
+        }
         if ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
             $self->mrest_declare_status( explanation => 'DISPATCH_NOTHING_IN_TSRANGE',
                 args => [ 'attendance intervals' ] 
@@ -1535,7 +1599,7 @@ sub _handler_get_interval {
 }
 
 
-=head2 handler_interval_new
+=head3 handler_interval_new
 
 Handler for 'POST interval/new'
 
@@ -1566,7 +1630,7 @@ sub handler_interval_new {
 }
 
 
-=head2 handler_post_interval_iid
+=head3 handler_post_interval_iid
 
 Handler for 'POST interval/iid'.
 
@@ -1592,14 +1656,64 @@ sub handler_post_interval_iid {
     return $fail unless $int;
 
     # - additional ACL check
-    return $fail unless acl_check_iid_lid( $self, $int->eid );
+    if ( ! acl_check_is_me( $self, 'eid' => $int->eid ) ) {
+        $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+        return $fail;
+    }
 
     # - perform the operation
     return shared_update_intlock( $self, $int, $context->{'request_entity'} );
 }
 
 
-=head2 handler_interval_iid
+=head3 handler_get_interval_iid
+
+Handler for 'GET interval/iid/:iid' resource.
+
+=cut
+
+sub handler_get_interval_iid {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__. "::handler_get_interval_iid" ); 
+
+    my $context = $self->context;
+
+    # first pass
+    if ( $pass == 1 ) {
+
+        # - get IID
+        my $iid = $self->context->{'mapping'}->{'iid'};
+        return 0 unless $iid;
+
+        # - is there an interval with this IID?
+        my $int = shared_first_pass_lookup( $self, 'IID', $iid );
+        return 0 unless $int;
+
+        # - additional ACL check
+        if ( 
+                ! acl_check_is_me( $self, 'eid' => $int->eid ) and
+                ! acl_check_is_my_report( $self, 'eid' => $int->eid ) 
+           ) 
+        {
+            $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+            return 0;
+        }
+
+        $context->{'stashed_interval_object'} = $int;
+        return 1;
+    }
+
+    # second pass
+    my $int = $context->{'stashed_interval_object'};
+    my $method = $context->{'method'};
+    if ( $method eq 'GET' ) {
+        return $CELL->status_ok( 'DISPATCH_INTERVAL_FOUND', payload => $int );
+    }
+    die "AAGAGAGGGGGGGGGGHHGHGHKD! method is " . ( $method || "undef" );
+}
+
+
+=head3 handler_interval_iid
 
 Handler for 'interval/iid/:iid' resource.
 
@@ -1623,7 +1737,10 @@ sub handler_interval_iid {
         return 0 unless $int;
 
         # - additional ACL check
-        return 0 unless acl_check_iid_lid( $self, $int->eid );
+        if ( ! acl_check_is_me( $self, 'eid' => $int->eid ) ) {
+            $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+            return 0;
+        }
 
         $context->{'stashed_interval_object'} = $int;
         return 1;
@@ -1632,14 +1749,12 @@ sub handler_interval_iid {
     # second pass
     my $int = $context->{'stashed_interval_object'};
     my $method = $context->{'method'};
-    if ( $method eq 'GET' ) {
-        return $CELL->status_ok( 'DISPATCH_INTERVAL_FOUND', payload => $int );
-    } elsif ( $method =~ m/^(PUT)|(POST)$/ ) {
+    if ( $method =~ m/^(PUT)|(POST)$/ ) {
         return shared_update_intlock( $self, $int, $context->{'request_entity'} );
     } elsif ( $method eq 'DELETE' ) {
         return $int->delete( $context );
     }
-    die "AAGAGAGGGGGGGGGGHHGHGHKD!";
+    die "AAGAGAGGGGGGGGGGHHGHGHKD! method is " . ( $method || "undef" );
 }
 
 
@@ -1734,117 +1849,7 @@ sub handler_get_interval_summary {
 =head2 Lock handlers
 
 
-=head3 handler_get_lock_eid
-
-Handler for 'GET lock/eid/:eid/:tsrange'
-
-#FIXME: implement a configurable limit on the tsrange
-
-=cut
-
-sub handler_get_lock_eid {
-    my ( $self, $pass ) = @_;
-    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_lock_eid " ); 
-
-    my $context = $self->context;
-
-    # first pass
-    if ( $pass == 1 ) {
-        my $status = App::Dochazka::REST::Model::Lock::fetch_by_eid_and_tsrange(
-            $context->{'dbix_conn'},
-            $context->{'mapping'}->{'eid'}, 
-            $context->{'mapping'}->{'tsrange'} 
-        );
-        if ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
-            $self->mrest_declare_status( explanation => 'DISPATCH_NOTHING_IN_TSRANGE',
-                args => [ 'lock intervals' ] 
-            );
-            return 0;
-        }
-        $context->{'stashed_lock_status'} = $status;
-        return 1;
-    }
-
-    # second pass
-    return $context->{'stashed_lock_status'};
-}
-
-
-=head3 handler_get_lock_nick
-
-Handler for 'GET lock/nick/:nick/:tsrange'
-
-#FIXME: implement a configurable limit on the tsrange
-
-=cut
-
-sub handler_get_lock_nick {
-    my ( $self, $pass ) = @_;
-    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_lock_nick " ); 
-
-    my $context = $self->context;
-
-    # first pass
-    if ( $pass == 1 ) {
-        my $emp = shared_first_pass_lookup( $self, 'nick', $self->context->{'mapping'}->{'nick'} );
-        return 0 unless $emp;
-        my $status = App::Dochazka::REST::Model::Lock::fetch_by_eid_and_tsrange(
-            $context->{'dbix_conn'},
-            $emp->eid,
-            $context->{'mapping'}->{'tsrange'} 
-        );
-        if ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
-            $self->mrest_declare_status( explanation => 'DISPATCH_NOTHING_IN_TSRANGE',
-                args => [ 'lock intervals' ] 
-            );
-            return 0;
-        }
-        $context->{'stashed_lock_status'} = $status;
-        return 1;
-    }
-
-    # second pass
-    return $context->{'stashed_lock_status'};
-}
-
-
-=head3 handler_get_lock_self
-
-Handler for 'GET lock/self/:tsrange'
-
-#FIXME: implement a configurable limit on the tsrange
-
-=cut
-
-sub handler_get_lock_self {
-    my ( $self, $pass ) = @_;
-    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_lock_self " ); 
-
-    my $context = $self->context;
-
-    # first pass
-    if ( $pass == 1 ) {
-        my $status = App::Dochazka::REST::Model::Lock::fetch_by_eid_and_tsrange(
-            $context->{'dbix_conn'},
-            $context->{'current'}->{'eid'}, 
-            $context->{'mapping'}->{'tsrange'} 
-        );
-        if ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
-            $self->mrest_declare_status( explanation => 'DISPATCH_NOTHING_IN_TSRANGE',
-                args => [ 'lock intervals' ] 
-            );
-            return 0;
-        }
-        $context->{'stashed_lock_status'} = $status;
-        return 1;
-    }
-
-    # second pass
-    return $context->{'stashed_lock_status'};
-}
-
-
-=head2 handler_interval_new
+=head3 handler_lock_new
 
 Handler for 'POST lock/new'
 
@@ -1875,7 +1880,7 @@ sub handler_lock_new {
 }
 
 
-=head2 handler_post_lock_lid
+=head3 handler_post_lock_lid
 
 Handler for 'POST lock/lid'.
 
@@ -1901,14 +1906,64 @@ sub handler_post_lock_lid {
     return $fail unless $lock;
 
     # - additional ACL check
-    return $fail unless acl_check_iid_lid( $self, $lock->eid );
+    if ( ! acl_check_is_me( $self, 'eid' => $lock->eid ) ) {
+        $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+        return $fail;
+    }
 
     # - perform the operation
     return shared_update_intlock( $self, $lock, $context->{'request_entity'} );
 }
 
 
-=head2 handler_lock_lid
+=head3 handler_get_lock_lid
+
+Handler for 'GET lock/lid/:lid' resource.
+
+=cut
+
+sub handler_get_lock_lid {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__. "::handler_get_lock_lid" ); 
+
+    my $context = $self->context;
+
+    # first pass
+    if ( $pass == 1 ) {
+
+        # - get LID
+        my $lid = $self->context->{'mapping'}->{'lid'};
+        return 0 unless $lid;
+
+        # - is there a lock with this LID?
+        my $lock = shared_first_pass_lookup( $self, 'LID', $lid );
+        return 0 unless $lock;
+
+        # - additional ACL check
+        if ( 
+                ! acl_check_is_me( $self, 'eid' => $lock->eid ) and
+                ! acl_check_is_my_report( $self, 'eid' => $lock->eid ) 
+           ) 
+        {
+            $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+            return 0;
+        }
+
+        $context->{'stashed_lock_object'} = $lock;
+        return 1;
+    }
+
+    # second pass
+    my $lock = $context->{'stashed_lock_object'};
+    my $method = $context->{'method'};
+    if ( $method eq 'GET' ) {
+        return $CELL->status_ok( 'DISPATCH_LOCK_FOUND', payload => $lock );
+    }
+    die "AAGAGAGGGGGGGGGGHHGHGHKD! method is " . ( $method || "undef" );
+}
+
+
+=head3 handler_lock_lid
 
 Handler for 'lock/lid/:lid' resource.
 
@@ -1932,7 +1987,13 @@ sub handler_lock_lid {
         return 0 unless $lock;
 
         # - additional ACL check
-        return 0 unless acl_check_iid_lid( $self, $lock->eid );
+        if ( 
+                ! acl_check_is_me( $self, 'eid' => $lock->eid )
+           ) 
+        {
+            $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+            return 0;
+        }
 
         $context->{'stashed_lock_object'} = $lock;
         return 1;
@@ -1941,14 +2002,12 @@ sub handler_lock_lid {
     # second pass
     my $lock = $context->{'stashed_lock_object'};
     my $method = $context->{'method'};
-    if ( $method eq 'GET' ) {
-        return $CELL->status_ok( 'DISPATCH_LOCK_FOUND', payload => $lock );
-    } elsif ( $method =~ m/^(PUT)|(POST)$/ ) {
+    if ( $method =~ m/^(PUT)|(POST)$/ ) {
         return shared_update_intlock( $self, $lock, $context->{'request_entity'} );
     } elsif ( $method eq 'DELETE' ) {
         return $lock->delete( $context );
     }
-    die "AAGAGAGGGGGGGGGGHHGHGHKD!";
+    die "AAGAGAGGGGGGGGGGHHGHGHKD! method is " . ( $method || "undef" );
 }
 
 
