@@ -54,11 +54,11 @@ App::Dochazka::REST::ResourceDefs - Resource definitions
 
 =head1 VERSION
 
-Version 0.458
+Version 0.462
 
 =cut
 
-our $VERSION = '0.458';
+our $VERSION = '0.462';
 my $tsrange_validation = qr/^[[(].*,.*[])]$/;
 my $term_validation = qr/^[[:alnum:]_][[:alnum:]_-]*$/;
 my $date_validation = qr/^\d{2,4}-\d{1,2}-\d{1,2}$/;
@@ -1687,7 +1687,7 @@ EOH
     {
         parent => 'interval',
         handler => {
-            GET => 'handler_interval_iid',
+            GET => 'handler_get_interval_iid',
             PUT => 'handler_interval_iid',
             DELETE => 'handler_interval_iid',
         },
@@ -1918,7 +1918,7 @@ EOH
     {
         parent => 'lock',
         handler => {
-            GET => 'handler_lock_lid',
+            GET => 'handler_get_lock_lid',
             PUT => 'handler_lock_lid',
             DELETE => 'handler_lock_lid',
         },
