@@ -40,7 +40,7 @@ use strict;
 use warnings;
 
 use App::CELL qw( $CELL $log $site );
-use App::Dochazka::REST::ACL qw( acl_check_is_me );
+use App::Dochazka::REST::ACL qw( acl_check_is_me acl_check_is_my_report );
 use App::Dochazka::REST::ConnBank qw( conn_status );
 use App::Dochazka::REST::Model::Activity;
 use App::Dochazka::REST::Model::Employee;
@@ -67,11 +67,11 @@ App::Dochazka::REST::Dispatch::Shared - Shared dispatch functions
 
 =head1 VERSION
 
-Version 0.458
+Version 0.462
 
 =cut
 
-our $VERSION = '0.458';
+our $VERSION = '0.462';
 
 my $fail = $CELL->status_not_ok;
 
@@ -256,7 +256,11 @@ sub shared_get_employee {
         #
         # ACL check - passerby can only look up him- or herself
         #
-        if ( ! acl_check_is_me( $d_obj, ( lc $key ) => $value ) ) {
+        if ( 
+                ! acl_check_is_my_report( $d_obj, ( lc $key ) => $value ) and
+                ! acl_check_is_me( $d_obj, ( lc $key ) => $value ) 
+           ) 
+        {
             $d_obj->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
             return 0;
         }
