@@ -68,11 +68,11 @@ App::Dochazka::REST::Test - Test helper functions
 
 =head1 VERSION
 
-Version 0.458
+Version 0.462
 
 =cut
 
-our $VERSION = '0.458';
+our $VERSION = '0.462';
 
 
 
@@ -102,7 +102,7 @@ our @EXPORT = qw(
     gen_activity gen_employee gen_interval gen_lock
     gen_privhistory gen_schedhistory gen_schedule
     test_sql_success test_sql_failure do_select_single
-    test_employee_list
+    test_employee_list get_aid_by_code
 );
 
 
@@ -773,4 +773,15 @@ sub test_employee_list {
     is_deeply( $status->payload, $nicks );
 }
 
+sub get_aid_by_code {
+    my ( $test, $code ) = @_;
+    my $status = req( $test, 200, 'root', 'GET', "activity/code/$code" );
+    is( $status->level, 'OK' );
+    is( $status->code, 'DISPATCH_ACTIVITY_FOUND' );
+    ok( $status->{'payload'} );
+    ok( $status->{'payload'}->{'aid'} );
+    is( $status->{'payload'}->{'code'}, uc( $code ) );
+    return $status->{'payload'}->{'aid'};
+}
+
 1;
